@@ -57,7 +57,12 @@ class VersionAdmin(admin.ModelAdmin):
     def _autoregister(self, model, follow=None):
         """Registers a model with reversion, if required."""
         if model._meta.proxy:
-            raise RegistrationError("Proxy models cannot be used with django-reversion, register the parent class instead")
+            raise RegistrationError(
+                "{model} is a proxy model, and cannot be used with django-reversion, register the parent class ({model_parent}) instead.".format(  # noqa
+                    model=model.__name__,
+                    model_parent=', '.join(
+                        [x.__name__ for x in model._meta.parents.keys()])
+                ))
         if not self.revision_manager.is_registered(model):
             follow = follow or []
             for parent_cls, field in model._meta.parents.items():
@@ -81,6 +86,7 @@ class VersionAdmin(admin.ModelAdmin):
             for field in self.model._meta.virtual_fields:
                 if isinstance(field, GenericRelation) and field.rel.to == inline_model and field.object_id_field_name == ct_fk_field and field.content_type_field_name == ct_field:
                     follow_field = field.name
+                    break
         elif issubclass(inline, options.InlineModelAdmin):
             inline_model = inline.model
             fk_name = inline.fk_name
@@ -88,7 +94,8 @@ class VersionAdmin(admin.ModelAdmin):
                 for field in inline_model._meta.fields:
                     if isinstance(field, (models.ForeignKey, models.OneToOneField)) and issubclass(self.model, field.rel.to):
                         fk_name = field.name
-            if not inline_model._meta.get_field(fk_name).rel.is_hidden():
+                        break
+            if fk_name and not inline_model._meta.get_field(fk_name).rel.is_hidden():
                 accessor = inline_model._meta.get_field(fk_name).related.get_accessor_name()
                 follow_field = accessor
         return inline_model, follow_field
@@ -276,6 +283,7 @@ class VersionAdmin(admin.ModelAdmin):
                 form_validated = False
                 new_object = obj
             prefixes = {}
+
             for FormSet, inline in zip(self.get_formsets(request, new_object),
                                        self.get_inline_instances(request)):
                 prefix = FormSet.get_default_prefix()
@@ -302,7 +310,7 @@ class VersionAdmin(admin.ModelAdmin):
                     formset.save_m2m()
                 change_message = _("Reverted to previous version, saved on %(datetime)s") % {"datetime": localize(version.revision.date_created)}
                 self.log_change(request, new_object, change_message)
-                self.message_user(request, _(u'The %(model)s "%(name)s" was reverted successfully. You may edit it again below.') % {"model": force_text(opts.verbose_name), "name": force_text(obj)})
+                self.message_user(request, _('The %(model)s "%(name)s" was reverted successfully. You may edit it again below.') % {"model": force_text(opts.verbose_name), "name": force_text(obj)})
                 # Redirect to the model change form.
                 if revert:
                     return HttpResponseRedirect("../../")
@@ -379,7 +387,7 @@ class VersionAdmin(admin.ModelAdmin):
             assert False
         return render_to_response(form_template, context, template.RequestContext(request))
     
-    @transaction.commit_on_success
+    @transaction.atomic
     def recover_view(self, request, version_id, extra_context=None):
         """Displays a form that can recover a deleted model."""
         # check if user has change or add permissions for model
@@ -391,7 +399,7 @@ class VersionAdmin(admin.ModelAdmin):
         context.update(extra_context or {})
         return self.render_revision_form(request, obj, version, context, recover=True)
         
-    @transaction.commit_on_success
+    @transaction.atomic
     def revision_view(self, request, object_id, version_id, extra_context=None):
         """Displays the contents of the given revision."""
         # check if user has change or add permissions for model
@@ -444,7 +452,7 @@ class VersionMetaAdmin(VersionAdmin):
     the last version that was saved.
     """
         
-    def queryset(self, request):
+    def get_queryset(self, request):
         """Returns the annotated queryset."""
         content_type = ContentType.objects.get_for_model(self.model)
         pk = self.model._meta.pk
@@ -452,7 +460,7 @@ class VersionMetaAdmin(VersionAdmin):
             version_table_field = "object_id_int"
         else:
             version_table_field = "object_id"
-        return super(VersionMetaAdmin, self).queryset(request).extra(
+        return super(VersionMetaAdmin, self).get_queryset(request).extra(
             select = {
                 "date_modified": """
                     SELECT MAX(%(revision_table)s.date_created)
@@ -469,7 +477,7 @@ class VersionMetaAdmin(VersionAdmin):
             },
             select_params = (content_type.id,),
         )
-        
+    
     def get_date_modified(self, obj):
         """Displays the last modified date of the given object, typically for use in a change list."""
         return localize(obj.date_modified)
