@@ -3,7 +3,7 @@
 from __future__ import unicode_literals
 
 import operator, sys
-from functools import wraps, reduce
+from functools import wraps, reduce, partial
 from threading import local
 from weakref import WeakValueDictionary
 
@@ -11,35 +11,35 @@ from django.contrib.contenttypes.models import ContentType
 from django.core import serializers
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.signals import request_finished
-from django.db import models, DEFAULT_DB_ALIAS, connection
-from django.db.models import Q, Max
+from django.db import models, connection, transaction
+from django.db.models import Q, Max, get_model
 from django.db.models.query import QuerySet
-from django.db.models.signals import post_save, pre_delete
+from django.db.models.signals import post_save
 from django.utils.encoding import force_text
 
 from reversion.models import Revision, Version, has_int_pk, pre_revision_commit, post_revision_commit
 
 
 class VersionAdapter(object):
-    
+
     """Adapter class for serializing a registered model."""
-    
+
     # Fields to include in the serialized data.
     fields = ()
-    
+
     # Fields to exclude from the serialized data.
     exclude = ()
-    
+
     # Foreign key relationships to follow when saving a version of this model.
     follow = ()
-    
+
     # The serialization format to use.
     format = "json"
-    
+
     def __init__(self, model):
         """Initializes the version adapter."""
         self.model = model
-        
+
     def get_fields_to_serialize(self):
         """Returns an iterable of field names to serialize in the version data."""
         opts = self.model._meta
@@ -50,7 +50,7 @@ class VersionAdapter(object):
                 yield field.name
             else:
                 yield field.attname
-    
+
     def get_followed_relations(self, obj):
         """Returns an iterable of related models that should be included in the revision data."""
         for relationship in self.follow:
@@ -78,11 +78,11 @@ class VersionAdapter(object):
                     relationship = relationship,
                     related = related,
                 ))
-    
+
     def get_serialization_format(self):
         """Returns the serialization format to use."""
         return self.format
-        
+
     def get_serialized_data(self, obj):
         """Returns a string of serialized data for the given obj."""
         return serializers.serialize(
@@ -90,11 +90,10 @@ class VersionAdapter(object):
             (obj,),
             fields = list(self.get_fields_to_serialize()),
         )
-        
+
     def get_version_data(self, obj, db=None):
         """Creates the version data to be saved to the version model."""
         object_id = force_text(obj.pk)
-        db = db or DEFAULT_DB_ALIAS
         content_type = ContentType.objects.db_manager(db).get_for_model(obj)
         if has_int_pk(obj.__class__):
             object_id_int = int(obj.pk)
@@ -111,20 +110,20 @@ class VersionAdapter(object):
 
 
 class RevisionManagementError(Exception):
-    
+
     """Exception that is thrown when something goes wrong with revision managment."""
 
-          
+
 class RevisionContextManager(local):
-    
+
     """Manages the state of the current revision."""
-    
+
     def __init__(self):
         """Initializes the revision state."""
         self.clear()
         # Connect to the request finished signal.
         request_finished.connect(self._request_finished_receiver)
-    
+
     def clear(self):
         """Puts the revision manager back into its default state."""
         self._objects = {}
@@ -135,31 +134,31 @@ class RevisionContextManager(local):
         self._meta = []
         self._ignore_duplicates = False
         self._db = None
-    
+
     def is_active(self):
         """Returns whether there is an active revision for this thread."""
         return bool(self._stack)
-    
+
     def is_managing_manually(self):
         """Returns whether this revision context has manual management enabled."""
         self._assert_active()
         return self._stack[-1]
-    
+
     def _assert_active(self):
         """Checks for an active revision, throwning an exception if none."""
         if not self.is_active():
             raise RevisionManagementError("There is no active revision for this thread")
-        
+
     def start(self, manage_manually=False):
         """
         Begins a revision for this thread.
-        
+
         This MUST be balanced by a call to `end`.  It is recommended that you
         leave these methods alone and instead use the revision context manager
-        or the `create_on_success` decorator.
+        or the `create_revision` decorator.
         """
         self._stack.append(manage_manually)
-    
+
     def end(self):
         """Ends a revision for this thread."""
         self._assert_active()
@@ -188,11 +187,11 @@ class RevisionContextManager(local):
         """Marks this revision as broken, so should not be commited."""
         self._assert_active()
         self._is_invalid = True
-        
+
     def is_invalid(self):
         """Checks whether this revision is invalid."""
         return self._is_invalid
-    
+
     def add_to_context(self, manager, obj, version_data):
         """Adds an object to the current revision."""
         self._assert_active()
@@ -206,7 +205,7 @@ class RevisionContextManager(local):
     def get_db(self):
         """Returns the current DB alias being used."""
         return self._db
-    
+
     def set_db(self, db):
         """Sets the DB alias to use."""
         self._db = db
@@ -215,39 +214,39 @@ class RevisionContextManager(local):
         """Sets the current user for the revision."""
         self._assert_active()
         self._user = user
-    
+
     def get_user(self):
         """Gets the current user for the revision."""
         self._assert_active()
         return self._user
-        
+
     def set_comment(self, comment):
         """Sets the comments for the revision."""
         self._assert_active()
         self._comment = comment
-    
+
     def get_comment(self):
         """Gets the current comment for the revision."""
         self._assert_active()
         return self._comment
-        
+
     def add_meta(self, cls, **kwargs):
         """Adds a class of meta information to the current revision."""
         self._assert_active()
         self._meta.append((cls, kwargs))
-    
+
     def set_ignore_duplicates(self, ignore_duplicates):
         """Sets whether to ignore duplicate revisions."""
         self._assert_active()
         self._ignore_duplicates = ignore_duplicates
-        
+
     def get_ignore_duplicates(self):
         """Gets whether to ignore duplicate revisions."""
         self._assert_active()
         return self._ignore_duplicates
-    
+
     # Signal receivers.
-    
+
     def _request_finished_receiver(self, **kwargs):
         """
         Called at the end of a request, ensuring that any open revisions
@@ -256,13 +255,13 @@ class RevisionContextManager(local):
         """
         while self.is_active():
             self.end()
-    
+
     # High-level context management.
-    
+
     def create_revision(self, manage_manually=False):
         """
         Marks up a block of code as requiring a revision to be created.
-        
+
         The returned context manager can also be used as a decorator.
         """
         return RevisionContext(self, manage_manually)
@@ -276,11 +275,11 @@ class RevisionContext(object):
         """Initializes the revision context."""
         self._context_manager = context_manager
         self._manage_manually = manage_manually
-    
+
     def __enter__(self):
         """Enters a block of revision management."""
         self._context_manager.start(self._manage_manually)
-        
+
     def __exit__(self, exc_type, exc_value, traceback):
         """Leaves a block of revision management."""
         try:
@@ -288,23 +287,13 @@ class RevisionContext(object):
                 self._context_manager.invalidate()
         finally:
             self._context_manager.end()
-        
+
     def __call__(self, func):
         """Allows this revision context to be used as a decorator."""
         @wraps(func)
         def do_revision_context(*args, **kwargs):
-            self.__enter__()
-            exception = False
-            try:
-                try:
-                    return func(*args, **kwargs)
-                except:
-                    exception = True
-                    if not self.__exit__(*sys.exc_info()):
-                        raise
-            finally:
-                if not exception:
-                    self.__exit__(None, None, None)
+            with self:
+                return func(*args, **kwargs)
         return do_revision_context
 
 
@@ -313,28 +302,28 @@ revision_context_manager = RevisionContextManager()
 
 
 class RegistrationError(Exception):
-    
+
     """Exception thrown when registration with django-reversion goes wrong."""
-   
-   
+
+
 class RevisionManager(object):
-    
+
     """Manages the configuration and creation of revisions."""
-    
+
     _created_managers = WeakValueDictionary()
-    
+
     @classmethod
     def get_created_managers(cls):
         """Returns all created revision managers."""
         return list(cls._created_managers.items())
-    
+
     @classmethod
     def get_manager(cls, manager_slug):
         """Returns the manager with the given slug."""
         if manager_slug in cls._created_managers:
             return cls._created_managers[manager_slug]
         raise RegistrationError("No revision manager exists with the slug %r" % manager_slug)
-    
+
     def __init__(self, manager_slug, revision_context_manager=revision_context_manager):
         """Initializes the revision manager."""
         # Check the slug is unique for this revision manager.
@@ -351,19 +340,33 @@ class RevisionManager(object):
 
     # Registration methods.
 
+    def _registration_key_for_model(self, model):
+        meta = model._meta
+        return (
+            meta.app_label,
+            meta.model_name,
+        )
+
     def is_registered(self, model):
         """
         Checks whether the given model has been registered with this revision
         manager.
         """
-        return model in self._registered_models
-    
+        return self._registration_key_for_model(model) in self._registered_models
+
     def get_registered_models(self):
         """Returns an iterable of all registered models."""
-        return list(self._registered_models.keys())
-        
-    def register(self, model, adapter_cls=VersionAdapter, **field_overrides):
+        return [
+            get_model(*key)
+            for key
+            in self._registered_models.keys()
+        ]
+
+    def register(self, model=None, adapter_cls=VersionAdapter, **field_overrides):
         """Registers a model with this revision manager."""
+        # Return a class decorator if model is not given
+        if model is None:
+            return partial(self.register, adapter_cls=adapter_cls, **field_overrides)
         # Prevent multiple registration.
         if self.is_registered(model):
             raise RegistrationError("{model} has already been registered with django-reversion".format(
@@ -371,38 +374,45 @@ class RevisionManager(object):
             ))
         # Prevent proxy models being registered.
         if model._meta.proxy:
-            raise RegistrationError("Proxy models cannot be used with django-reversion, register the parent class instead")
+            raise RegistrationError(
+                "{model} is a proxy model, and cannot be used with django-reversion, register the parent class ({model_parent}) instead.".format(  # noqa
+                    model=model.__name__,
+                    model_parent=', '.join(
+                        [x.__name__ for x in model._meta.parents.keys()])
+                ))
         # Perform any customization.
         if field_overrides:
             adapter_cls = type(adapter_cls.__name__, (adapter_cls,), field_overrides)
         # Perform the registration.
         adapter_obj = adapter_cls(model)
-        self._registered_models[model] = adapter_obj
+        self._registered_models[self._registration_key_for_model(model)] = adapter_obj
         # Connect to the post save signal of the model.
         post_save.connect(self._post_save_receiver, model)
-    
+        return model
+
     def get_adapter(self, model):
         """Returns the registration information for the given model class."""
         if self.is_registered(model):
-            return self._registered_models[model]
+            return self._registered_models[self._registration_key_for_model(model)]
         raise RegistrationError("{model} has not been registered with django-reversion".format(
             model = model,
         ))
-        
+
     def unregister(self, model):
         """Removes a model from version control."""
         if not self.is_registered(model):
             raise RegistrationError("{model} has not been registered with django-reversion".format(
                 model = model,
             ))
-        del self._registered_models[model]
+        del self._registered_models[self._registration_key_for_model(model)]
         post_save.disconnect(self._post_save_receiver, model)
-    
+
     def _follow_relationships(self, objects):
         """Follows all relationships in the given set of objects."""
         followed = set()
         def _follow(obj):
-            if obj in followed or obj.pk is None:
+            # Check the pk first because objects without a pk are not hashable
+            if obj.pk is None or obj in followed:
                 return
             followed.add(obj)
             adapter = self.get_adapter(obj.__class__)
@@ -411,18 +421,15 @@ class RevisionManager(object):
         for obj in objects:
             _follow(obj)
         return followed
-    
+
     def _get_versions(self, db=None):
         """Returns all versions that apply to this manager."""
-        db = db or DEFAULT_DB_ALIAS
         return Version.objects.using(db).filter(
             revision__manager_slug = self._manager_slug,
         ).select_related("revision")
-        
+
     def save_revision(self, objects, ignore_duplicates=False, user=None, comment="", meta=(), db=None):
         """Saves a new revision."""
-        # Get the db alias.
-        db = db or DEFAULT_DB_ALIAS
         # Adapt the objects to a dict.
         if isinstance(objects, (list, tuple)):
             objects = dict(
@@ -454,7 +461,7 @@ class RevisionManager(object):
                         if sorted(previous_versions) == sorted(all_serialized_data):
                             save_revision = False
             # Only save if we're always saving, or have changes.
-            if save_revision:                
+            if save_revision:
                 # Save a new revision.
                 revision = Revision(
                     manager_slug = self._manager_slug,
@@ -468,14 +475,15 @@ class RevisionManager(object):
                     versions = new_versions,
                 )
                 # Save the revision.
-                revision.save(using=db)
-                # Save version models.
-                for version in new_versions:
-                    version.revision = revision
-                Version.objects.using(db).bulk_create(new_versions)
-                # Save the meta information.
-                for cls, kwargs in meta:
-                    cls._default_manager.db_manager(db).create(revision=revision, **kwargs)
+                with transaction.atomic(using=db):
+                    revision.save(using=db)
+                    # Save version models.
+                    for version in new_versions:
+                        version.revision = revision
+                    Version.objects.using(db).bulk_create(new_versions)
+                    # Save the meta information.
+                    for cls, kwargs in meta:
+                        cls._default_manager.db_manager(db).create(revision=revision, **kwargs)
                 # Send the pre_revision_commit signal.
                 post_revision_commit.send(self,
                     instances = ordered_objects,
@@ -484,16 +492,15 @@ class RevisionManager(object):
                 )
                 # Return the revision.
                 return revision
-    
+
     # Revision management API.
-    
+
     def get_for_object_reference(self, model, object_id, db=None):
         """
         Returns all versions for the given object reference.
-        
+
         The results are returned with the most recent versions first.
         """
-        db = db or DEFAULT_DB_ALIAS
         content_type = ContentType.objects.db_manager(db).get_for_model(model)
         versions = self._get_versions(db).filter(
             content_type = content_type,
@@ -508,19 +515,19 @@ class RevisionManager(object):
             versions = versions.filter(object_id=object_id)
         versions = versions.order_by("-pk")
         return versions
-    
+
     def get_for_object(self, obj, db=None):
         """
         Returns all the versions of the given object, ordered by date created.
-        
+
         The results are returned with the most recent versions first.
         """
         return self.get_for_object_reference(obj.__class__, obj.pk, db)
-    
+
     def get_unique_for_object(self, obj, db=None):
         """
         Returns unique versions associated with the object.
-        
+
         The results are returned with the most recent versions first.
         """
         versions = self.get_for_object(obj, db)
@@ -531,7 +538,7 @@ class RevisionManager(object):
                 changed_versions.append(version)
             last_serialized_data = version.serialized_data
         return changed_versions
-    
+
     def get_for_date(self, object, date, db=None):
         """Returns the latest version of an object for the given date."""
         versions = self.get_for_object(object, db)
@@ -542,14 +549,13 @@ class RevisionManager(object):
             raise Version.DoesNotExist
         else:
             return version
-    
+
     def get_deleted(self, model_class, db=None, model_db=None):
         """
         Returns all the deleted versions for the given model class.
-        
+
         The results are returned with the most recent versions first.
         """
-        db = db or DEFAULT_DB_ALIAS
         model_db = model_db or db
         content_type = ContentType.objects.db_manager(db).get_for_model(model_class)
         live_pk_queryset = model_class._default_manager.db_manager(model_db).all().values_list("pk", flat=True)
@@ -578,9 +584,9 @@ class RevisionManager(object):
             deleted_version_pks = list(deleted_version_pks)
         # Return the deleted versions!
         return self._get_versions(db).filter(pk__in=deleted_version_pks).order_by("-pk")
-        
+
     # Signal receivers.
-        
+
     def _post_save_receiver(self, instance, **kwargs):
         """Adds registered models to the current revision, if any."""
         if self._revision_context_manager.is_active() and not self._revision_context_manager.is_managing_manually():
@@ -588,6 +594,6 @@ class RevisionManager(object):
             version_data = lambda: adapter.get_version_data(instance, self._revision_context_manager._db)
             self._revision_context_manager.add_to_context(self, instance, version_data)
 
-        
+
 # A shared revision manager.
 default_revision_manager = RevisionManager("default")
